use crate::objects::{get_objects, get_object_here, update_object_location, update_object_conditions};
use crate::models::Player;
use crate::database::establish_connection;
use diesel::prelude::*;
use crate::npc::get_npc_here;
use rand::Rng;

// get_player le o xogador polo nick
pub fn get_player(nome: String) -> Result<Player, diesel::result::Error> {
    use  crate::schema::player::dsl::*;
    let conn = &mut establish_connection();
    player
        .filter(nombre.eq(nome))
        .first(conn)
}

// create_player garda o xogador
pub fn create_player(newplayer: &Player) -> Result<usize, diesel::result::Error> { 
    use  crate::schema::player;
    let conn = &mut establish_connection();

    diesel::insert_into(player::table)
        .values(newplayer)
        .execute(conn)
}

// update_player actualiza o xogador
pub fn update_player(newplayer: &Player) -> Result<usize, diesel::result::Error> {
    use  crate::schema::player::dsl::*; // para ter o find no update
    let conn = &mut establish_connection();

    // FIXME: actualiza todos os rows...
    diesel::update(player::find(player, &newplayer.tag))
        .set(newplayer)  // co trait AsChangeSet podese actualizar a struct enteira
        .execute(conn)
}

// execute_search mostra os obxetos na ubicación do xogador
pub fn execute_search(player: &mut Player) -> String { 
    let mut output: String;
    let current = &player.location;
    let objetos = get_objects(current.to_string());

    if objetos.is_ok() && objetos.as_ref().unwrap().len() > 0 {
        output = String::from("En esta sala encuentras:\n");
        for objeto in objetos.unwrap() {
                let mut nombre = objeto.nombre;
                nombre.push_str("\n");
                output.push_str(&nombre);
        }
    } else {
        output = String::from("No encuentras ningún objeto en esta sala.")
    }

    output
}

// execute_inventory abre o inventario
pub fn execute_inventory(player: &mut Player) -> String {
    let mut output: String;
    let current = &player.tag;
    let objetos = get_objects(current.to_string());

    if objetos.is_ok() && objetos.as_ref().unwrap().len() > 0 {
        output = String::from("Portas los siguientes objetos:\n");
        for objeto in objetos.unwrap() {
                let mut nombre = objeto.nombre;
                nombre.push_str("\n");
                output.push_str(&nombre);
        }
    } else {
        output = String::from("No portas ningún objeto.")
    }

    output
}

// execute_pick recolle un obxeto
pub fn execute_pick(player: &mut Player, nombre: &str) -> String { 
    let output: String;
    let location = &player.location;
    let obxeto = get_object_here(location.to_string(), nombre.to_string());

    if obxeto.is_ok() {
        let res = update_object_location(
            obxeto.as_ref().unwrap().tag.clone(), 
            player.tag.to_string());

        if res.is_ok() && res.unwrap() > 0 {
            output = format!("{} ahora está en tu inventario.", obxeto.unwrap().nombre);
        } else {
            output = format!("Error cogiendo {}.", obxeto.unwrap().nombre);
        }
    } else {
        output = String::from("No encuentro lo que quieres coger.");
    }

    output
}

// execute_drop tira un obxeto
pub fn execute_drop(player: &mut Player, nombre: &str) -> String {
    let output: String;
    let location = &player.tag;
    let obxeto = get_object_here(location.to_string(), nombre.to_string());

    if obxeto.is_ok() {
        let res = update_object_location(
            obxeto.as_ref().unwrap().tag.clone(),
            player.location.to_string()); //tirao aquí

        if res.is_ok() && res.unwrap() > 0 {
            output = format!("Tiras {}.", obxeto.unwrap().nombre);
        } else {
            output = format!("Error tirando {}.", obxeto.unwrap().nombre);
        }
    } else {
        output = String::from("No encuentro lo que quieres tirar.");
    }

    output
}

// execute_kill intenta matar un npc
pub fn execute_kill(player: &mut Player, nombre: &str) -> String {
    let mut output= String::from("");
    let location = &player.location;
    let npc = get_npc_here(location.to_string(), nombre.to_string());

    if npc.is_ok() {
        // non hai iniciativa, comeza o xogador
        let mut malo = npc.unwrap();
        while malo.hp >= 0 && player.hp >= 0 {
            // turno do xogador:
            let tohit = rand::thread_rng().gen_range(1..20) + player.ba;
            if tohit >= malo.ca {
                let damage = rand::thread_rng().gen_range(1..6);
                malo.hp = malo.hp - damage;
                let tmp = format!("Golpeas a {} por {} de daño, le quedan {} pg.", malo.nombre, damage, malo.hp);
                output = format!("{}\n{}", output, tmp);
            } else {
                let tmp = format!("No consigues golpear a {}.", malo.nombre);
                output = format!("{}\n{}", output, tmp);
            }

            // turno do malo
            let tohit = rand::thread_rng().gen_range(1..20) + malo.ba;
            if tohit >= player.ca {
                let damage = rand::thread_rng().gen_range(1..6);
                player.hp = player.hp - damage;
                let tmp = format!("{} te golpea por {} de daño, te quedan {} pg.", malo.nombre, damage, player.hp);
                output = format!("{}\n{}", output, tmp);
            } else {
                let tmp = format!("{} no consigue herirte.", malo.nombre);
                output = format!("{}\n{}", output, tmp);
            }
            print!("{}",output); //FIXME si saco esto, non o concatena coa salida
        }

        let resolucion: String;
        if malo.hp <= 0 {
            resolucion = format!("{} ha muerto!", malo.nombre);
        } else if player.hp <= 0 {
            resolucion = format!("{} ha muerto!", player.nombre);
        } else {
            resolucion = String::from("No se que ha pasado. Alguien debería estar muerto.");
        }
        output = format!("{}\n{}", output, resolucion);

    } else {
        output = String::from("No encuentro lo que quieres matar.");
    }

    output
}

//execute_open cambia a condición 'close' a 'open' de un obxeto nesta sala polo nome
pub fn execute_close(player: &mut Player, nombre: &str) -> String {
    let mut output = String::from("");
    let location = &player.location;
    let objeto = get_object_here(location.to_string(), nombre.to_string());

    if objeto.is_ok() {
        let mut obj = objeto.unwrap();
        println!("{:?}", obj);

        if obj.tipo == "salida" {
            for (pos, condicion) in obj.condiciones.iter().enumerate() {
                if condicion.as_ref().unwrap() == "open" {
                    output = String::from("Está abierta.");
                    // actualizar a salida en sentido contrario
                    // implementar get_salida_opuesta() para actualizar o sentido contario tamén

                    obj.condiciones[pos] = Some("closed".to_string());
                    let res = update_object_conditions(location.to_string(), obj.condiciones);

                    // non actualiza, da ok pero actualiza 0 rows
                    match res {
                        Ok(n) => println!("actualizados {} rows", n),
                        Err(error) => println!("Error actualizando: {:?}", error),
                    };

                    break;
                } else if condicion.as_ref().unwrap() == "closed" {
                    output = String::from("Ya está cerrada.");
                    break;
                } else {
                    output = String::from("");
                }
            }
        } else {
            // é un item
            output = String::from("No se puede abrir.");
        }
    } else {
        output = String::from("No se encuentra aquí.")
    }
    output
}
